/**
 * 
 */
package de.mcs.tools.sps.emulator.emulator.holtek;

import de.mcs.tools.sps.emulator.EmulatorInput;
import de.mcs.tools.sps.emulator.EmulatorInternals;
import de.mcs.tools.sps.emulator.EmulatorOutput;
import de.mcs.tools.sps.emulator.SPSCommandDataImpl;

/**
 * @author w.klaas
 *
 */
public class SPSCommandDataWait extends SPSCommandDataImpl {

  private static final long[] WAIT_TIMES = new long[] {
      // @formatter-off
      1, 2, 5, 10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000, 20000, 30000, 60000
      // @formatter-on
  };

  private static final String[] WAIT_NAMES = new String[] {
      // @formatter-off
      "1ms", "2ms", "5ms", "10ms", "20ms", "50ms", "100ms", "200ms", "500ms", "1s", "2s", "5s", "10s", "20s", "30s",
      "60s"
      // @formatter-on
  };

  static String getName(int i) {
    return String.format("Wait %s", WAIT_NAMES[i]);
  }

  static String getMnemonic(int i) {
    return String.format("D", WAIT_NAMES[i]).toUpperCase();
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * de.mcs.tools.sps.SPSCommandDataImpl#doWork(de.mcs.tools.sps.EmulatorInput,
   * de.mcs.tools.sps.EmulatorInternals)
   */
  @Override
  public EmulatorOutput doWork(EmulatorInput input, EmulatorInternals internals, EmulatorOutput output) {
    byte data = getData();
    if ((data >= 0) && (data < 16)) {
      try {
        internals.setDelay(WAIT_TIMES[data]);
        long delay = WAIT_TIMES[data];
        if (delay <= 100) {
          Thread.sleep(delay);
        } else {
          long count = delay / 100;
          while (count > 0) {
            count--;
            Thread.sleep(100);
            internals.setDelay(count * 100);
          }
        }
      } catch (InterruptedException e) {
        e.printStackTrace();
      }
    } else {
      throw new IllegalArgumentException(String.format("Wait time %d illegal", data));
    }
    internals.incAddress(1);
    return output;
  }

}
