package de.mcs.tools.sps.mnemonic;

import static org.junit.jupiter.api.Assertions.assertEquals;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import de.mcs.tools.sps.exceptions.SyntaxError;

class TestSingleMnemonics {

  @Test
  void testSWAP() throws SyntaxError {
    SWAP mno = new SWAP("SWAP");
    assertEquals(0x50, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      SWAP mno1 = new SWAP("SWAP akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SWAP mno1 = new SWAP("SWAP 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      SWAP mno1 = new SWAP("SWAP :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testPUSH() throws SyntaxError {
    PUSH mno = new PUSH("PUSH");
    assertEquals(0x5f, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      PUSH mno1 = new PUSH("PUSH akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      PUSH mno1 = new PUSH("PUSH 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      PUSH mno1 = new PUSH("PUSH :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testPOP() throws SyntaxError {
    POP mno = new POP("POP");
    assertEquals(0x6f, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      POP mno1 = new POP("POP akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      POP mno1 = new POP("POP 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      POP mno1 = new POP("POP :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testINC() throws SyntaxError {
    INC mno = new INC("INC");
    assertEquals(0x71, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      INC mno1 = new INC("INC akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      INC mno1 = new INC("INC 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      INC mno1 = new INC("INC :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testDEC() throws SyntaxError {
    DEC mno = new DEC("DEC");
    assertEquals(0x72, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      DEC mno1 = new DEC("DEC akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      DEC mno1 = new DEC("DEC 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      DEC mno1 = new DEC("DEC :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testADD() throws SyntaxError {
    ADD mno = new ADD("ADD");
    assertEquals(0x73, mno.getByte());

    Assertions.assertThrows(SyntaxError.class, () -> {
      ADD mno1 = new ADD("DEC akfhaskh");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      ADD mno1 = new ADD("ADD 12");
      mno1.checkArgument();
    });

    Assertions.assertThrows(SyntaxError.class, () -> {
      ADD mno1 = new ADD("ADD :label");
      mno1.checkArgument();
    });
  }

  @Test
  void testSUB() throws SyntaxError {
    SUB mno = new SUB("SUB");
    assertEquals(0x74, mno.getByte());
  }

  @Test
  void testMUL() throws SyntaxError {
    MUL mno = new MUL("MUL");
    assertEquals(0x75, mno.getByte());
  }

  @Test
  void testDIV() throws SyntaxError {
    DIV mno = new DIV("DIV");
    assertEquals(0x76, mno.getByte());
  }

  @Test
  void testAND() throws SyntaxError {
    AND mno = new AND("AND");
    assertEquals(0x77, mno.getByte());
  }

  @Test
  void testOR() throws SyntaxError {
    OR mno = new OR("OR");
    assertEquals(0x78, mno.getByte());
  }

  @Test
  void testXOR() throws SyntaxError {
    XOR mno = new XOR("XOR");
    assertEquals(0x79, mno.getByte());
  }

  @Test
  void testNOT() throws SyntaxError {
    NOT mno = new NOT("NOT");
    assertEquals(0x7a, mno.getByte());
  }

  @Test
  void testMOD() throws SyntaxError {
    MOD mno = new MOD("MOD");
    assertEquals(0x7b, mno.getByte());
  }

  @Test
  void testBYTE() throws SyntaxError {
    BYTE mno = new BYTE("BYTE");
    assertEquals(0x7c, mno.getByte());
  }

  @Test
  void testBSUBA() throws SyntaxError {
    BSUBA mno = new BSUBA("BSUBA");
    assertEquals(0x7d, mno.getByte());
  }

  @Test
  void testSKIP0() throws SyntaxError {
    SKIP0 mno = new SKIP0("SKIP0");
    assertEquals(0xc0, mno.getByte());
  }

  @Test
  void testAGTB() throws SyntaxError {
    AGTB mno = new AGTB("AGTB");
    assertEquals(0xc1, mno.getByte());
  }

  @Test
  void testALTB() throws SyntaxError {
    ALTB mno = new ALTB("ALTB");
    assertEquals(0xc2, mno.getByte());
  }

  @Test
  void testAEQB() throws SyntaxError {
    AEQB mno = new AEQB("AEQB");
    assertEquals(0xc3, mno.getByte());
  }

  @Test
  void testPRG0() throws SyntaxError {
    PRG0 mno = new PRG0("PRG0");
    assertEquals(0xcc, mno.getByte());
  }

  @Test
  void testPRG1() throws SyntaxError {
    PRG1 mno = new PRG1("PRG1");
    assertEquals(0xce, mno.getByte());
  }

  @Test
  void testSEL0() throws SyntaxError {
    SEL0 mno = new SEL0("SEL0");
    assertEquals(0xcd, mno.getByte());
  }

  @Test
  void testSEL1() throws SyntaxError {
    SEL1 mno = new SEL1("SEL1");
    assertEquals(0xcf, mno.getByte());
  }

  @Test
  void testRTR() throws SyntaxError {
    RTR mno = new RTR("RTR");
    assertEquals(0xe0, mno.getByte());
  }

  @Test
  void testREST() throws SyntaxError {
    REST mno = new REST("REST");
    assertEquals(0xef, mno.getByte());
  }

  @Test
  void testTONE() throws SyntaxError {
    TONE mno = new TONE("TONE");
    assertEquals(0xf8, mno.getByte());
  }

  @Test
  void testPEND() throws SyntaxError {
    PEND mno = new PEND("PEND");
    assertEquals(0xff, mno.getByte());
  }

  @Test
  void testMnemonicFactory() throws SyntaxError {
    Mnemonic mnemonic = MnemonicFactory.getMnemonic("SWAP", 0);
    assertEquals(SWAP.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("PUSH", 0);
    assertEquals(PUSH.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("INC", 0);
    assertEquals(INC.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("DEC", 0);
    assertEquals(DEC.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("ADD", 0);
    assertEquals(ADD.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("SUB", 0);
    assertEquals(SUB.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("MUL", 0);
    assertEquals(MUL.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("DIV", 0);
    assertEquals(DIV.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("AND", 0);
    assertEquals(AND.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("OR", 0);
    assertEquals(OR.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("XOR", 0);
    assertEquals(XOR.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("NOT", 0);
    assertEquals(NOT.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("MOD", 0);
    assertEquals(MOD.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("BYTE", 0);
    assertEquals(BYTE.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("BSUBA", 0);
    assertEquals(BSUBA.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("SKIP0", 0);
    assertEquals(SKIP0.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("AGTB", 0);
    assertEquals(AGTB.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("ALTB", 0);
    assertEquals(ALTB.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("AEQB", 0);
    assertEquals(AEQB.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("PRG0", 0);
    assertEquals(PRG0.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("PRG1", 0);
    assertEquals(PRG1.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("SEL0", 0);
    assertEquals(SEL0.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("SEL1", 0);
    assertEquals(SEL1.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("RTR", 0);
    assertEquals(RTR.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("REST", 0);
    assertEquals(REST.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("TONE", 0);
    assertEquals(TONE.class, mnemonic.getClass());

    mnemonic = MnemonicFactory.getMnemonic("PEND", 0);
    assertEquals(PEND.class, mnemonic.getClass());
  }
}
